/*
 *
 * Flashing LEDS using two tasks
 * Pressing blue button -> Interrupt -> Buttontask runs sends data on USART3 (ST_LINK or J-Link)
 *
 *
 *
 *
 *
 *
 *
 */
#include "main.h"
#include "cmsis_os.h"
#include "usb_device.h"
#include "board.h"
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

#include "demo.h"
#define STARTTASK_PRIO 3
#define STARTTASK_DELAY 100
#define STARTTASK_STACKSIZE 128

#define LEDTASK1_PRIO 3
#define LEDTASK1_DELAY 1000
#define LEDTASK1_STACKSIZE 128

#define LEDTASK2_PRIO 4
#define LEDTASK2_DELAY 55
#define LEDTASK2_STACKSIZE 128

#define BUTTONTASK_PRIO 4
#define BUTTONTAS_DELAY 55
#define BUTTONTASK_STACKSIZE 128

static void LEDblink1(void *pvParameters );
static void LEDblink2(void *pvParameters );
static void StartTask(void *pvParameters );
static void ButtonTask(void *pvParameters );

TIM_HandleTypeDef htim6;
UART_HandleTypeDef huart3;
SemaphoreHandle_t  LEDMutex;
TaskHandle_t  TaskToNotify=NULL;
#ifdef TRC_USE_TRACEALYZER_RECORDER
traceString LED1Chn;
#endif


int main(void)
{
	HAL_Init();
	SystemClock_Config();
#ifdef TRC_USE_TRACEALYZER_RECORDER
#if (TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_SNAPSHOT)
    vTraceEnable(TRC_START);
#else
    vTraceEnable(TRC_INIT);
#endif
#endif
    MX_GPIO_Init();
    MX_USART3_UART_Init();
    MX_TIM6_Init();

    LEDMutex = xSemaphoreCreateMutex();

#if (TRC_USE_TRACEALYZER_RECORDER == 1)
    vTraceSetSemaphoreName(LEDMutex,"LED_Mutex");
    LED1Chn = xTraceRegisterString("LED1Status");
#endif

    xTaskCreate( StartTask, "Startup", STARTTASK_STACKSIZE, NULL, STARTTASK_PRIO, NULL );


#ifdef TRC_USE_TRACEALYZER_RECORDER
    vStartDemoApplication();
#endif


    /* Start scheduler */
    vTaskStartScheduler();

    while (1)
    {
    }
}

static void StartTask(void *pvParameters )
{
    /* Remove compiler warning about unused parameter. */
	( void ) pvParameters;


    xTaskCreate( LEDblink1, "LED1", LEDTASK1_STACKSIZE, NULL, LEDTASK1_PRIO, NULL );
	xTaskCreate( LEDblink2, "LED2", LEDTASK2_STACKSIZE, NULL, LEDTASK2_PRIO, NULL );
	xTaskCreate( ButtonTask, "Button", BUTTONTASK_STACKSIZE, NULL, BUTTONTASK_PRIO, &TaskToNotify );
	MX_USB_DEVICE_Init();


    for(int n=0;n<10;n++)
    {
        vTaskDelay(STARTTASK_DELAY);
        SetLED(LED2,LED_ON);
        vTaskDelay(STARTTASK_DELAY);
        SetLED(LED2,LED_OFF);
    }
    vTaskDelete(NULL);
}


static void LEDblink1(void *pvParameters )
{
	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;

    while(1)
	{
		xSemaphoreTake(LEDMutex,portMAX_DELAY );
        SetLED(LED1,LED_ON);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1)
            vTracePrintF(LED1Chn, "LED:%d",1);
        #endif
		vTaskDelay(LEDTASK1_DELAY);
        SetLED(LED1,LED_OFF);
		xSemaphoreGive(LEDMutex);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1)
            vTracePrintF(LED1Chn, "LED:%d",0);
        #endif
		vTaskDelay(LEDTASK1_DELAY);
	}
}


static void LEDblink2(void *pvParameters )
{
	/* Remove compiler warning about unused parameter. */
	( void ) pvParameters;
	while(1)
	{
		xSemaphoreTake(LEDMutex,portMAX_DELAY );
        SetLED(LED1,LED_ON);
    	#if (TRC_USE_TRACEALYZER_RECORDER == 1)
            vTracePrintF(LED1Chn, "LED:%d",1);
        #endif
		vTaskDelay(LEDTASK2_DELAY);
        SetLED(LED1,LED_OFF);
		xSemaphoreGive(LEDMutex);
        #if (TRC_USE_TRACEALYZER_RECORDER == 1)
     		vTracePrintF(LED1Chn, "LED:%d",0);
        #endif
		vTaskDelay(LEDTASK2_DELAY);
	}
}

void HAL_GPIO_EXTI_Callback(uint16_t GPIO_Pin)
{
	BaseType_t xHigherPriorityTaskWoken = pdFALSE;

	if(GPIO_Pin==GPIO_PIN_13)
	{
		 vTaskNotifyGiveFromISR( TaskToNotify, &xHigherPriorityTaskWoken );
	}
	portYIELD_FROM_ISR( xHigherPriorityTaskWoken );
}
static void ButtonTask(void *pvParameters)
{
	char *msg = "Do not press the button!\n\r";


	while(1)
	{
		ulTaskNotifyTake( pdTRUE, portMAX_DELAY);
		HAL_UART_Transmit(&huart3, (uint8_t*)msg, strlen(msg), 0xFFFF);
	}
}


